//source: utilities.cpp
//Last changed 02.08.2006
// functions used in the definitions of classes

#include<math.h>
#include"utilities.h"


//----------------skipline utility for reading files-----------
void skipline(ifstream &file)
{
	int code;
	if(!file) return;
	else
	{
		code=file.get();
		while(code!=10) code=file.get();
	}
};
//----------------j30 formatting utility to write numbers cleanly----
//this utility creates a printing space of 30 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in SimpleCfg::save(ifstream &file)

ostream &j30(ostream &os) {
	os.width(30);
	os.fill(' ');
 	return os;
}

//----------------j20 formatting utility to write numbers cleanly----
//this utility creates a printing space of 20 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in SimpleCfg::save(ifstream &file)

ostream &j20(ostream &os) {
	os.width(20);
	os.fill(' ');
 	return os;
}
//----------------j10 formatting utility to write numbers cleanly----
//this utility creates a printing space of 10 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in SimpleCfg::save(ifstream &file)

ostream &j10(ostream &os) {
	os.width(10);
	os.fill(' ');
 	return os;
}

//----------------j4 formatting utility to write numbers cleanly----
//this utility creates a printing space of 4 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in test of normalisation

ostream &j4(ostream &os) {
	os.width(4);
	os.fill(' ');
 	return os;
}


//----------------j7 formatting utility to write numbers cleanly----
//this utility creates a printing space of 7 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in test of normalisation

ostream &j7(ostream &os) {
	os.width(7);
	os.fill(' ');
 	return os;
}

//----------------j13 formatting utility to write numbers cleanly----
//this utility creates a printing space of 13 characters filled with spaces
//for writing the next output ( with << )
//justifying and adjusting the precision of real numbers is done externally
//see use in SimpleCfg::save(ifstream &file)

ostream &j13(ostream &os) {
	os.width(13);
	os.fill(' ');
 	return os;
}

//----------------random number generator from numerical recipies--------
//returns a number flatly distributed between 0 and 1
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran1(long &idum)
{
        int j;
        long k;
        static long iy=0;
        static long iv[NTAB];
        double temp;
        
        if(idum <=0 || !iy) {
                if (-idum < 1) idum=1;
                else idum = -idum;
                for (j=NTAB+7;j>=0;j--) {
                        k=idum/IQ;
                        idum=IA*(idum-k*IQ)-IR*k;
                        if (idum<0) idum += IM;
                        if (j<NTAB) iv[j]=idum;
                }
                iy=iv[0];
        }
        k=idum/IQ;
        idum=IA*(idum-k*IQ)-IR*k;
        if (idum <0) idum += IM;
        j=iy/NDIV;
        iy=iv[j];
        iv[j]=idum;
        if ((temp=AM*iy) > RNMX) return RNMX;
        else return temp;
}


//converts a number's digit into their character representation
//for example: integer=4012 to *char="4012"; integer=-521 to *char"-521" 
char *IntToStr(int number)
{
	int i,digits,length,remainder;
	int pos_number;
	char *conv_number;

	if (number==0)
	{
		conv_number=new char[2];
		conv_number[0]='0';
		conv_number[1]='\0';
		return(conv_number);
	}
		
	pos_number=abs(number);
	digits=(int)log10(pos_number)+1;
	if (number<0)
		length=digits+2;//to have a place for the minus 
	else
		length=digits+1;

	conv_number=new char[length];//the last place is for theterminating character
	if (number<0)
		*conv_number='-';
	*(conv_number+length-1)='\0';//just to put the terminating character to its place,

	//this cycle is cutting the last digit, and places it into its proper place in the character array
	for (i=length-2;i>=length-digits-1;i--)
	{
		remainder=(pos_number % 10);
		switch (remainder)
		{
			case 0:
			{
				conv_number[i]='0';
				break;
			}
			case 1:
			{
				conv_number[i]='1';
				break;
			}
			case 2:
			{
				conv_number[i]='2';
				break;
			}
			case 3:
			{
				conv_number[i]='3';
				break;
			}
			case 4:
			{
				conv_number[i]='4';
				break;
			}
			case 5:
			{
				conv_number[i]='5';
				break;
			}
			case 6:
			{
				conv_number[i]='6';
				break;
			}
			case 7:
			{
				conv_number[i]='7';
				break;
			}
			case 8:
			{
				conv_number[i]='8';
				break;
			}
			case 9:
			{
				conv_number[i]='9';
				break;
			}


		}//end switch
		pos_number-=remainder;
		pos_number/=10;//cut the last digit
	}
	return(conv_number);
};

//-------checking the file status after reading--------
void CheckReadFileState(ifstream &file, const char *routine_name)
{
	if (file.rdstate() & file.failbit)//there was some error during reading
	{
		if (file.rdstate() & file.eofbit)
		{	
			cout<<"****WARNING****"<<endl;
			cout<<routine_name<<endl;
			cout<<"End of file was reached during loading!"<<endl;
		}
		else
		{
			cout<<"****WARNING****"<<endl;
			cout<<routine_name<<endl;
			cout<<"Loading failed, possibly due to wrong file format!"<<endl;
		}
		cout<<"Loading was not successful, cannot run this way, exiting"<<endl;
		cleanexit();//loading was not successful
	}
	else
	{
		if (file.rdstate() & file.eofbit)//end of file was reached after the last required data, clear rdstate
			file.clear(ios::goodbit);//set it back to goodbit, it is needed, if the file variable is used again
	}

};

//------------------checking the file status, if the file is already open for reading--------------
int CheckFileState(ifstream &file,const  char *routine_name)
{

	if (file.is_open()==0 || !file)   //verification that file is open
	{
		cout<<"****WARNING****"<<endl;
		cout<<routine_name<<" file is not open for reading!"<<endl;
		file.clear(ios::goodbit);//set it back to goodbit, it is needed, if the file variable is used again
		return(0);//the file is not open 
		
	}
	else
		return(1);//the file is open
};

//------------------checking the file status, if the file is already open for saving--------------
int CheckFileState(ofstream &file, const char *routine_name)
{
	if (file.is_open()==0 || !file)   //verification that file is open
	{
		cout<<"****WARNING****"<<endl;
		cout<<routine_name<<" file could not be opened for saving!"<<endl;
		cout<<"Saving will be skipped, running continues."<<endl;
		cout<<"If the file is used by another application, close it, so next saving can be performed!"<<endl;
		file.clear(ios::goodbit);//set it back to goodbit, it is needed, if the file variable is used again
			return(0);//the file is not open 
		
	}
	else
		return(1);//the file is open
};

//------------------checking the file status after opening it for saving----------------
void OpenFile(ofstream &file, char *filename, const char *routine_name,int flag)
{
	//flag: 0:text mode, 1: binary mode
	
	int repeat_status;
	char key;

	
	do 
	{
		repeat_status=0;//do not repeat opening by default
		if (flag)
			file.open(filename,ios::binary);//open it in binary mode
		else
			file.open(filename);//open it in text mode 

		if (file.is_open()==0 || !file)   //verification that file is open
		{
			repeat_status=1;
			cout<<"****WARNING****"<<endl;
			cout<<routine_name<<" - file could not be opened for saving!"<<endl;
			cout<<"If the file is used by another application, close it first."<<endl;
			cout<<"Try again? (Press any key!)"<<endl;
			cin>>key;
			file.clear(ios::goodbit);//set it back to goodbit, it is needed, if the file variable is used again
			
		};
	}
	while (repeat_status);

};

//Sorting is done with the Oosterwal algorithm for an integer array
void O_Sort(int length, int *array)
{
	int i,step,temp;
	double SngFib,SngPhi;

	SngPhi = 0.78;//Define phi value
	SngFib=length*SngPhi;//Set initial real step size
	step = int(SngFib);//Set initial integer step size

	while (step>0)
	{

		for (i=0;i<length-step;i++)//Range of lower cell
		{
			if (array[i] > array[i+step]) //Compare cells
			{
				temp=array[i];
				array[i]=array[i+step];//Swap Cells
				array[i+step]=temp;
			}
		}

		SngFib = SngFib * SngPhi;//Decrease the Real step size
		step = int(SngFib);//Set the integer step value

	}
};
